<?php

namespace Ulmus\Api;

use Ulmus, Ulmus\Query\QueryFragmentInterface;

class RequestBuilder implements Ulmus\QueryBuilder\QueryBuilderInterface
{
    public Ulmus\QueryBuilder\QueryBuilderInterface $parent;

    public RequestBuilder\Filter $filters;

    public RequestBuilder\UrlParameter $urlParameters;

    protected int $parameterIndex = 0;

    protected array $queryParamStack = [];

    public function values(array $dataset) : self
    {
        $this->addValues(array_values($dataset));

        return $this;
    }

    public function where(\Stringable|string $field, mixed $value,) : self
    {
        if ( empty($this->filters) && ! $this->getFragment(RequestBuilder\Filter::class) ) {
            $this->filters = new RequestBuilder\Filter();
            $this->push($this->filters);
        }

        $this->filters->add($field, $value);

        return $this;
    }

    public function bindUrl(\Stringable|string $field, mixed $value,) : self
    {
        if ( empty($this->urlParameters) && ! $this->getFragment(RequestBuilder\UrlParameter::class) ) {
            $this->urlParameters = new RequestBuilder\UrlParameter();
            $this->push($this->urlParameters);
        }

        $this->urlParameters->add($field, $value);

        return $this;
    }


    /*public function notWhere($field, mixed $value) : self
    {
        return $this->filters($field, $value);
    }*/

    public function limit(int $value) : self
    {
        /*if ( null === $limit = $this->getFragment(Ulmus\Query\Limit::class) ) {
            $limit = new Query\Limit();
            $this->push($limit);
        }

        $limit->set($value);
*/
        return $this;
    }

    public function offset(int $value) : self
    {
       /*if ( null === $offset = $this->getFragment(Ulmus\Query\Offset::class) ) {
            $offset = new Ulmus\Query\Offset();
            $this->push($offset);
        }

        $offset->set($value);*/

        return $this;
    }

    public function orderBy(string $field, ? string $direction = null) : self
    {
        return $this;
    }

    public function push(QueryFragmentInterface $queryFragment) : self
    {
        $this->queryParamStack[] = $queryFragment;

        return $this;
    }

    public function pull(QueryFragmentInterface $queryFragment) : self
    {
        return array_shift($this->queryParamStack);
    }

    public function render(bool $skipToken = false) : array
    {
        $stack = [];

        foreach($this->queryParamStack as $fragment) {
            $stack = array_merge($stack, (array) $fragment->render($skipToken));
        }

        return $stack;
    }

    public function reset() : void
    {
        unset($this->filters);
    }

    public function getFragment(string $class, int $index = 0) : ? QueryFragmentInterface
    {
        foreach($this->queryParamStack as $item) {
            if ( get_class($item) === $class ) {
                if ( $index-- === 0 ) {
                    return $item;
                }
            }
        }

        return null;
    }

    public function removeFragment(QueryFragmentInterface|array|\Stringable|string $fragment) : void
    {
        foreach($this->queryParamStack as $key => $item) {
            if ( $item === $fragment ) {
                unset($this->queryParamStack[$key]);
            }
        }
    }

    public function __toString() : string
    {
        return $this->render();
    }
}
